unit HallSel;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, HotImage, StdCtrls;

type
  TfrmHallSel = class(TForm)
    ScrollBox1: TScrollBox;
    HotImage1: THotImage;
    Panel1: TPanel;
    Label1: TLabel;
    Button1: TButton;
    Button2: TButton;

    procedure HotImage1GetHintText(Sender: TObject; Spot: TSpot;
      var Text: String);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure HotImage1Paint(Sender: TObject);
    procedure HotImage1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Button1Click(Sender: TObject);
    procedure ScrollBox1Resize(Sender: TObject);
    procedure ScrollBox1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    { Private declarations }
    FSpots:TList; //list of currently selected spots

    procedure LoadData;
    procedure DrawSelection(ASpot:TSpot);
    procedure SetSelection(ASpot:TSpot);
    procedure AddSelection(ASPot:TSpot);
    procedure CancelSelection;
  public
    { Public declarations }
  end;

var
  frmHallSel: TfrmHallSel;

implementation

uses sellist;

{$R *.DFM}

{procedure for drawing selection }
procedure TfrmHallSel.DrawSelection(ASpot:TSpot);
var
  i:integer;
  Offst:TPoint;
begin
 Offst:=Point(0,0);
//note - here is painting directly to the screen, not bitmap
 With HotImage1.Canvas do
  begin
    Pen.Mode:=pmNotXor; //inverted drawing
    Pen.Width:=4;
    Pen.Color:=clGray;
    Pen.Style:=psInsideFrame;
    Brush.Style:=bsClear;
  end;
 if ASpot<>nil then ASpot.DrawShape(HotImage1.Canvas,Offst,false)
 else
   for i:=0 to FSpots.Count-1 do
     TSpot(FSpots[i]).DrawShape(HotImage1.Canvas,Offst,false);

end;

procedure TfrmHallSel.SetSelection(ASpot:TSpot);
begin
 DrawSelection(nil);
 FSpots.Clear;
 if assigned(ASpot) then
 begin
   FSpots.Add(ASpot);
   DrawSelection(ASpot);
 end;
end;

procedure TfrmHallSel.AddSelection(ASPot:TSpot);
var
 i:integer;
begin
 if ASpot=nil then exit;
 if FSpots.Count=0 then SetSelection(ASpot) else

 With HotImage1 do
 begin
  DrawSelection(ASpot);
  i:=FSpots.IndexOf(ASpot);
  if i>=0 then FSpots.Delete(i)
  else FSpots.Add(ASpot);
 end;
end;

procedure TfrmHallSel.CancelSelection;
begin
 DrawSelection(nil);
 FSpots.Clear; //remove all selection
end;


procedure TfrmHallSel.LoadData;

procedure AddSite(X,Y:integer;const SiteNo:string;
                   SiteID:integer;BrushColor:TColor;Hot:boolean);
var
  R:TRect;
  Spot:TSpot;
begin
 R:=Rect(X,Y,X+40,y+40);
 {To draw on picture(not to screen directly)-
       use PictureCanvas instead of Canvas}
 With HotImage1.PictureCanvas do
 begin
  Brush.Color:=BrushColor;
  FillRect(R);
  DrawText(Handle,PChar(SiteNo),
           length(SiteNo),R,DT_CENTER or DT_VCENTER or DT_SINGLELINE);
  if Hot then
  begin
   Spot:=TSpot.Create(skRectangle);
   Spot.SpotID:=SiteID;
   Spot.HintText:=SiteNo;
   Spot.BoundsRect:=R;
   HotImage1.HotSpots.AddSpot(Spot);
  end;
 end;
end;
{better to know total picture rectangle BEFORE loading data - it's vy time-consuming
  operation to do that calculation each time after all data loaded.
  Store that size in database.
   if you have a problem with that - write me }
var
  PictHeight,
  PictWidth: integer;

begin
  PictHeight := 400;
  PictWidth := 500;


  With HotImage1 do
  begin
    CancelSelection;
    Picture.Graphic := nil; //invalidate all picture
    BoundsRect := Bounds(0, 0, PictWidth, PictHeight);//Set Size of HotImage
    PictureCanvas.Brush.Color := clBlack;
    PictureCanvas.FillRect(BoundsRect);
  {you can load your background picture instead}
 end;
{ assume loading from database}

  AddSite(10, 10, '1', 1{perhaps Row num}, clWhite, true);
  AddSite(60, 10, '2', 1, clWhite, true);
  AddSite(110,10, '3', 1, clWhite,true);
  AddSite(160,10, '4', 1, clRed,false);
  AddSite(210,10, '5', 1, clWhite,true);
  AddSite(260,10, '6', 1, clWhite,true);
  AddSite(260,10, '7', 1, clWhite,true);
  AddSite(310,10, '8', 1, clWhite,true);
  AddSite(360,10, '9', 1, clWhite,true);

  AddSite(25, 60, '1', 2, clWhite,true);
  AddSite(75, 60, '2', 2, clWhite,true);
  AddSite(125,60, '3', 2, clWhite,true);
  AddSite(175,60, '4', 2, clGreen,false);
  AddSite(225,60, '5', 2, clGreen,false);
  AddSite(275,60, '6', 2, clWhite,true);
  AddSite(325,60, '7', 2, clWhite,true);



end;



{just demonstration of OnGetHintText event handler}
procedure TfrmHallSel.HotImage1GetHintText(Sender: TObject; Spot: TSpot;
  var Text: String);
begin
 if assigned(Spot) then
   Text := Format('Raw #%d, site "%s"',[Spot.SPotID,Spot.HintText]);
end;

procedure TfrmHallSel.FormCreate(Sender: TObject);
begin
 FSpots:=TList.Create; //list of pointers
 {remove flickering during resizing}
 With ScrollBox1 do
   SetClassLong(Handle, GCL_STYLE, GetClassLong(Handle, GCL_STYLE) and
                 (not (CS_HREDRAW or CS_VREDRAW)));
 LoadData;
end;

procedure TfrmHallSel.FormDestroy(Sender: TObject);
begin
  FSpots.Free;
end;

procedure TfrmHallSel.HotImage1Paint(Sender: TObject);
begin
  DrawSelection(nil);
end;

procedure TfrmHallSel.HotImage1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
 with HotImage1 do
  if (ssCtrl in Shift) then  AddSelection(SelectedSpot)
   else SetSelection(SelectedSpot);

end;

procedure TfrmHallSel.ScrollBox1MouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if not (ssCtrl in Shift) then CancelSelection;
end;



procedure TfrmHallSel.Button1Click(Sender: TObject);
var
  i: integer;
begin
 with TfrmSelList.Create(Application)do
 try
  For i := 0 to FSpots.Count - 1 do
  With TSpot(FSpots[i])do
   ListBox1.Items.Add(Format('Raw=%d, Num=%s',[SpotID,HintText]));
  ShowModal;
 finally
   Free;
 end;
end;

function Max(X,Y:integer):integer;
begin
 result := X;
 if Y > result then result := Y;
end;

procedure TfrmHallSel.ScrollBox1Resize(Sender: TObject);
var
 BR: TRect;
begin

  With HotImage1 do
  begin
    BR := Bounds(Max(0, (ScrollBox1.ClientWidth - Width) div 2),
                 Max(0, (ScrollBox1.ClientHeight - Height) div 2),
                 Width, Height);
    ScrollBox1.VertScrollBar.Position := 0;
    ScrollBox1.HorzScrollBar.Position := 0;
    BoundsRect := BR;

    {Remove flickering }
    ValidateRect(ScrollBox1.handle, @BR);
    Refresh;
  end;
end;

end.
