program emf2hmt;

{$APPTYPE CONSOLE}
uses

  windows,
  sysutils,
  classes,
  graphics,
  hotimage,
  hotparse;


function EnumMFRecords(DC: HDC;
                       var lpHTable: THANDLETABLE;
                       const lpEMR: TENHMETARECORD;
                       nObj: integer;
                       lpData: THotSpots): Integer; stdcall;

procedure NormalizeRect(var R: TRect);
var
  Tmp: Integer;
begin
  If R.Left> R.Right then
  begin
    Tmp := R.Left;
    R.Left := R.Right;
    R.Right := Tmp;
  end;
  If R.Top > R.Bottom then
  begin
    Tmp := R.Top;
    R.Top := R.Bottom;
    R.Bottom := Tmp;  
  end;
  inc(R.Left);  
  inc(R.Top);
  inc(R.Right, 2);
  inc(R.Bottom, 2);

end;
type
{$IFDEF VER90}
    TEMRPolyline16_= packed record
       emr: TEMR;
       rclBounds: TRect; { Inclusive-inclusive bounds in device units}
       cpts: DWORD;
       apts: array[0..0] of TSmallPoint;
     end;
{$ELSE}
  TEMRPolyline16_ = TEMRPolyline16;

{$ENDIF}
  PEMRPolyline16_ = ^TEMRPolyline16_;

var Spot: TSpot;
    I: DWORD;
    tmpPtPtr1, tmpPtPtr2: ^TPoint;
begin
  Spot := nil;

  if lpEMR.iType in [1, 9, 10, 11, 12, 14, 17, 31, 32, 33, 34, 105, 106, 110] then
    PlayEnhMetafileRecord(DC, lpHTable, lpEmr, nObj);




  case  lpEMR.iType of

    EMR_POLYGON:
    With PEMRPolyline(@lpEMR)^ do
    begin
      Spot := TSpot.Create(skPolyline);
      Spot.Count := cptl;
      Move(aptl, Spot.GetPointArray^, cptl*sizeof(TPoint));

    end;

    EMR_RECTANGLE:
    With PEMREllipse(@lpEMR)^ do
    begin
     Spot := TSpot.Create(skRectangle);
     Spot.BoundsRect := rclBox;
     NormalizeRect(PRect(Spot.GetPointArray)^);
    end;

    EMR_ELLIPSE:
    With PEMREllipse(@lpEMR)^ do
    begin
     Spot := TSpot.Create(skEllipse);
     Spot.BoundsRect := rclBox;
     NormalizeRect(PRect(Spot.GetPointArray)^);
    end;

    EMR_POLYLINE:
    With PEMRPolyline(@lpEMR)^ do
    begin
      tmpPtPtr1 := @aptl[cptl-1];
      tmpPtPtr2 := @aptl[0];
      if (tmpPtPtr1^.x = tmpPtPtr2^.x)and (tmpPtPtr1^.y = tmpPtPtr2^.y) then
      begin
        Spot := TSpot.Create(skPolyline);
        Spot.Count := Pred(cptl);
        Move(aptl, Spot.GetPointArray^, Pred(cptl)* sizeof(TPoint));
      end;

    end;


    EMR_POLYGON16:
    with PEMRPolyline16_(@lpEMR)^ do
    begin
     Spot := TSpot.Create(skPolyLine);
     for I := 0 to Pred(cpts) do
      Spot.AddPoint(SmallPointToPoint(apts[i]));
    end;


    EMR_POLYLINE16:
     with PEMRPolyline16_(@lpEMR)^ do
     begin
       if Cardinal(apts[cpts-1]) = Cardinal(apts[0]) then
       begin
        Spot := TSpot.Create(skPolyLine);
        for I := 0 to Pred(Pred(cpts)) do
          Spot.AddPoint(SmallPointToPoint(apts[i]));
       end;
     end;

 end;


 if Assigned(Spot) then
 begin
  LPtoDP(DC, Spot.GetPointArray^,Spot.Count);
  lpData.AddSpot(Spot);

 end;
  result:=1;
end;



var
  MF: TMetaFile;
  DC: hDC;
  HS: THotSpots;
  OutFN: String;

begin
  Writeln('Metafile to hotspots  conversion utility');
  Writeln('Copyright 1998 Miachin Alexander (MAA)');
  Writeln;

  If ParamCount<1 then
  begin
    Writeln('Usage: emf2hmt <filename.emf> [output file]');
    exit;
  end;

  MF := TMetaFile.Create;
  try
    MF.LoadFromFile(ParamStr(1));
    DC :=GetDC(0);
    HS:=THotSpots.Create(nil);
    try
      EnumEnhMetaFile(DC, MF.Handle, @EnumMFRecords, HS, Rect(0, 0, MF.Width, MF.Height));

      if HS.Count>0 then
      begin
        Writeln(HS.Count,' entries readed from EMF');
       if ParamCount >1 then OutFN := ParamStr(2)
        else 
          OutFN := ChangeFileExt(ParamStr(1),'.HMT');

        WriteHStoFile(OutFN, HS);
        Writeln('Data was written in: ', OutFN);
      end;

    finally
      ReleaseDC(0,DC);
      HS.Free;
    end;
  finally
    MF.Free;
  end;
end.